# -*- coding: utf-8 -*-

from .asetukset import *

DEBUG = CONFIG('DEBUG', cast=bool, default=True)
ALLOWED_HOSTS = CONFIG(
  'ALLOWED_HOSTS',
  cast=lambda x: list(map(str.strip, x.split(','))),
  default='*',
)
SECRET_KEY = 'epäjärjestelmällistyttämättömyydellänsäkäänköhän'



# -*- coding: utf-8 -*-

import importlib
import os
from pathlib import Path
from sys import version_info
import warnings

from decouple_multi import AutoConfig, UndefinedValueError


# Käytetään yhtenäistä entry_point-toteutusta, vaikka Python 3.9 ja 3.10
# eroavat tältä osin toisistaan.
if version_info >= (3, 10):
  from importlib.metadata import entry_points
  def ep_hakemisto(ep):
    # Huomaa, että `entry_point`-standardi ei hyväksy vinoviivoja
    # osana liitospistettä. Korvataan mahdolliset pisteet viivoilla.
    return ep.dist.locate_file(ep.value.replace('.', '/'))

else:
  # Ks. https://docs.python.org/3/library/importlib.metadata.html#entry-points.
  from importlib.metadata import entry_points as _entry_points
  def entry_points(*, group):
    return _entry_points().get(group, ())
  def ep_hakemisto(ep):
    from importlib.metadata import distributions
    ep_value = ep.value.replace('.', '/')
    for distribution in distributions():
      if (hakemisto := distribution.locate_file(ep_value)).exists():
        return hakemisto
    else:
      raise ValueError(f'Tiedostoa ei löydetty: {ep.value}')

  # else (Python < 3.10)


# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent

# Asetusparametrit.
CONFIG = AutoConfig(
  search_path=tuple(filter(None, (
    # Ympäristössä määritetty hakemisto.
    # Tunnistetaan myös mahdollinen `ENV_POLKU`-muuttuja.
    os.getenv('PROTONI', None) or os.getenv('ENV_POLKU', None),
    # Käytetään toissijaisesti nykyistä työhakemistoa.
    Path().resolve(),
    # Kolmannella sijalla käytetään asennettujen pakettien
    # määrittelemiä `decouple.asetukset`-hakemistoja.
    *(
      ep_hakemisto(entry_point)
      for entry_point in entry_points(
        group='decouple.asetukset'
      )
    ),
    # Huomaa, että `decouple` käyttää näiden lisäksi viimekätisenä
    # hakemistona käsillä olevan moduulin (__file__) sisältävää polkua.
  ))),
)

# IP-osoite ja portti, jossa `manage.py runserver`-palvelinta ajetaan.
# Oletus 127.0.0.1, 8000.
# Osoite ja portti poimitaan pilkulla eroteltuina.
# Mikäli vain toinen on annettu, tulkitaan numeerinen arvo portin
# numeroksi ja muu arvo IP-osoitteeksi.
RUNSERVER = CONFIG(
  'RUNSERVER',
  cast=lambda x: (
    # pylint: disable=undefined-variable
    ('127.0.0.1', 8000) if not x
    else (h_p[0], int(h_p[1])) if ',' in x and (h_p := x.split(','))
    else ('127.0.0.1', int(x)) if x.isdecimal()
    else (x, 8000)
  ),
  default=None,
)

# SSL-avain- ja -varmennetiedosto, joita käytetään https-kehityspalvelimen
# suojaamiseen.
RUNSERVER_SSL_KEYFILE=CONFIG(
  'RUNSERVER_SSL_KEYFILE',
  default=None,
)
RUNSERVER_SSL_CERTFILE=CONFIG(
  'RUNSERVER_SSL_CERTFILE',
  default=None,
)

# Oletusasetukset.
AUTH_PASSWORD_VALIDATORS = []

try:
  DATABASES = {
    'default': {
      'ENGINE': CONFIG('DB_ENGINE'),
    }
  }

except UndefinedValueError:
  pass

else:
  DATABASES['default'].update({
    'django.db.backends.sqlite3': lambda: {
      'NAME': CONFIG('DB_NAME', default=os.path.join(BASE_DIR, 'db.sqlite3')),
    },
    'django.db.backends.mysql': lambda: {
      'HOST': CONFIG('DB_HOST', default='127.0.0.1'),
      'PORT': CONFIG('DB_PORT', default=3306, cast=int),
      'NAME': CONFIG('DB_NAME'),
      'USER': CONFIG('DB_USER'),
      'PASSWORD': CONFIG('DB_PASSWORD', default=''),
      'OPTIONS': {
        'sql_mode': 'TRADITIONAL',
        'charset': 'utf8',
        'init_command': (
          'SET '
          'NAMES "utf8", '
          'sql_mode=STRICT_TRANS_TABLES,'
          'default_storage_engine=INNODB,'
          'character_set_connection=utf8,'
          'character_set_database=utf8,'
          'character_set_server=utf8,'
          'collation_connection=utf8_swedish_ci'
        ),
      },
    },
    'django.db.backends.postgresql': lambda: {
      'HOST': CONFIG('DB_HOST', default='127.0.0.1'),
      'PORT': CONFIG('DB_PORT', default=5432, cast=int),
      'NAME': CONFIG('DB_NAME'),
      'USER': CONFIG('DB_USER'),
      'PASSWORD': CONFIG('DB_PASSWORD', default=''),
    },
  }.get(DATABASES['default']['ENGINE'], lambda: {})())

INSTALLED_APPS = [
  *(
    f'django.contrib.{sovellus}'
    for sovellus in (
      'admin',
      'auth',
      'contenttypes',
      'sessions',
      'messages',
      'staticfiles',
      'humanize',
    )
    if sovellus in CONFIG(
      'DJANGO_SOVELLUKSET',
      cast=lambda x: x.split(','),
      default='',
    )
  ),
  'django.forms',
]
MIDDLEWARE = [
  'django.middleware.security.SecurityMiddleware',
  *((
    'django.contrib.sessions.middleware.SessionMiddleware',
  ) if 'django.contrib.sessions' in INSTALLED_APPS else ()),
  'django.middleware.locale.LocaleMiddleware',
  'django.middleware.common.CommonMiddleware',
  'django.middleware.csrf.CsrfViewMiddleware',
  *((
    'django.contrib.auth.middleware.AuthenticationMiddleware',
  ) if 'django.contrib.auth' in INSTALLED_APPS else ()),
  *((
    'django.contrib.messages.middleware.MessageMiddleware',
  ) if 'django.contrib.messages' in INSTALLED_APPS else ()),
  'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

# Käytetään istuntojen tallennukseen tietokantaa, mikäli se on
# määritetty ja muutoin evästeitä.
SESSION_ENGINE = CONFIG(
  'SESSION_ENGINE',
  default=(
    'django.contrib.sessions.backends.db' if 'DATABASES' in locals()
    else 'django.contrib.sessions.backends.signed_cookies'
  )
)

TEMPLATES = [
  {
    'BACKEND': 'django.template.backends.django.DjangoTemplates',
    'DIRS': [],
    'APP_DIRS': True,
    'OPTIONS': {
      'context_processors': [
        'django.template.context_processors.debug',
        'django.template.context_processors.request',
        *((
          'django.contrib.auth.context_processors.auth',
        ) if 'django.contrib.auth' in INSTALLED_APPS else ()),
        *((
          'django.contrib.messages.context_processors.messages',
        ) if 'django.contrib.messages' in INSTALLED_APPS else ()),
      ],
    },
  },
]

DEFAULT_AUTO_FIELD = 'django.db.models.AutoField'

ROOT_URLCONF = 'protoni.osoitteet'
WSGI_APPLICATION = 'protoni.wsgi.application'

LANGUAGE_CODE = 'fi-fi'
TIME_ZONE = CONFIG('TIME_ZONE', default='UTC')
USE_I18N = True
USE_TZ = CONFIG('USE_TZ', cast=bool, default=True)

STATIC_URL = '/static/'
STATIC_ROOT = CONFIG(
  'STATIC_ROOT',
  default=os.path.join('/', 'tmp', 'protoni', 'static')
)

MEDIA_URL = '/media/'
MEDIA_ROOT = CONFIG(
  'MEDIA_ROOT',
  default=os.path.join('/', 'tmp', 'protoni', 'media')
)

LOGIN_REDIRECT_URL = '/'
LOGOUT_REDIRECT_URL = '/'

# Sisäänkirjautumisosoite.
LOGIN_URL = '/kirjaudu/sisaan/'


LOGGING = {
  'version': 1,
  'disable_existing_loggers': False,
  'formatters': {
    'verbose': {
      'format': ('%(asctime)s [%(process)d] [%(levelname)s] '
            'pathname=%(pathname)s lineno=%(lineno)s '
            'funcname=%(funcName)s %(message)s'),
      'datefmt': '%Y-%m-%d %H:%M:%S'
    },
    'simple': {
      'format': '%(levelname)s %(message)s'
    }
  },
  'handlers': {
    'null': {
      'level': 'DEBUG',
      'class': 'logging.NullHandler',
    },
    'console': {
      'level': 'INFO',
      'class': 'logging.StreamHandler',
      'formatter': 'verbose'
    }
  },
  'loggers': {
    'django': {
      'handlers': ['console'],
      'level': 'DEBUG',
      'propagate': True,
    },
    'django.request': {
      'handlers': ['console'],
      'level': 'DEBUG',
      'propagate': False,
    },
  }
}


# Lataa asennetut sovellukset.
for entry_point in entry_points(group='django.sovellus'):
  INSTALLED_APPS.append(entry_point.value)


# Lataa asennetut asetuslaajennokset.
for entry_point in entry_points(group='django.asetukset'):
  spec = importlib.util.find_spec(entry_point.value)
  if spec is None:
    entry_point.load()
    warnings.warn(f'Virheellinen laajennos: {entry_point!r}')
    continue
  with open(spec.origin, encoding='utf-8') as laajennos_mod:
    # pylint: disable=exec-used
    try: exec(compile(laajennos_mod.read(), spec.origin, 'exec'))
    except ImportError: pass
  # for entry_point in pkg_resources.iter_entry_points



# -*- coding: utf-8 -*-

if not CONFIG('DDT', cast=bool, default=True):
  raise ImportError

import debug_toolbar
del debug_toolbar

INSTALLED_APPS += ['debug_toolbar']
MIDDLEWARE.insert(
  min((
    i for i, ohjain in enumerate(MIDDLEWARE)
    if ohjain in (
      'django.middleware.gzip.GZipMiddleware',
    )
  ), default=-1) + 1,
  'debug_toolbar.middleware.DebugToolbarMiddleware'
)
INTERNAL_IPS = ['127.0.0.1', '::1']



# -*- coding: utf-8 -*-

import django_hosts
del django_hosts

INSTALLED_APPS += ['django_hosts']
MIDDLEWARE.insert(
  min((
    i for i, ohjain in enumerate(MIDDLEWARE)
    if ohjain in (
      'django.middleware.gzip.GZipMiddleware',
    )
  ), default=-1) + 1,
  'django_hosts.middleware.HostsRequestMiddleware',
)
MIDDLEWARE.insert(
  len(MIDDLEWARE) + 1,
  'django_hosts.middleware.HostsResponseMiddleware',
)

ROOT_HOSTCONF = 'protoni.palvelimet'
DEFAULT_HOST = CONFIG('DJANGO_DEFAULT_HOST', default='<oletus>')



# -*- coding: utf-8 -*-

from importlib.metadata import entry_points
import sys
import warnings

from django.apps import apps
from django.conf import settings
from django.urls import include, path

from django_hosts import patterns, host


_entry_points = (
  # Ks. https://docs.python.org/3/library/importlib.metadata.html#entry-points.
  entry_points
  if sys.version_info >= (3, 10)
  else lambda *, group: entry_points().get(group, ())
)


_yleinen_osoitteisto = []
for entry_point in _entry_points(group='django.nakymat'):
  try:
    moduuli = entry_point.load()
  except (ImportError, AttributeError):
    pass
  else:
    _yleinen_osoitteisto.append(
      path(entry_point.name + '/', include(moduuli)),
    )
    # else
  # for entry_point in _entry_points

# Luo projektiosoitteiston (`osoitteet.py`) mukainen oletuspalvelin.
palvelimet = [
]

# Käy kukin rekisteröity palvelinnimiavaruus läpi.
for entry_point in _entry_points(group='django.palvelin'):
  # Poimi palvelimen nimi ja osoitteistomoduuli.
  nimi, moduuli = entry_point.name, entry_point.value

  # Käytetään osoitteiston nimiavaruutena sovelluksen nimeä.
  sovellus = apps.get_containing_app_config(moduuli)
  if sovellus is None:
    warnings.warn(f'Sovellusta ei löydy: {moduuli!r}', stacklevel=2)
    continue

  # Muodosta palvelinkohtainen osoitteisto yhdistelmänä seuraavista:
  # - nimetty, yksittäinen osoiteavaruus ja
  # - projektitasoiset, yleiset osoiteavaruudet.
  class PalvelinkohtainenOsoitteisto:
    urlpatterns = [
      path(
        '',
        include((moduuli, sovellus.name), namespace=sovellus.name),
      ),
      *_yleinen_osoitteisto
    ]
    # class PalvelinkohtainenOsoitteisto

  # Luo oma moduulinsa tästä osoitteistosta.
  moduuli += '.palvelin'
  sys.modules[moduuli] = PalvelinkohtainenOsoitteisto

  # Muodosta `<palvelin>.*`-määritys, joka hakee käyttämänsä
  # osoitteiston edellä asetetusta luettelosta.
  palvelimet.append(host(rf'^(.*[.])?{nimi}[.].*', moduuli, name=nimi))
  # for entry_point in _entry_points

if not any(palvelin.name == '<oletus>' for palvelin in palvelimet):
  palvelimet.append(
    host(r'', settings.ROOT_URLCONF, name='<oletus>'),
  )

host_patterns = patterns('', *palvelimet)



# -*- coding: utf-8 -*-

from importlib.metadata import entry_points
from sys import version_info

from django.conf import settings
from django.urls import path, include
from django.views.generic import RedirectView


_entry_points = (
  # Ks. https://docs.python.org/3/library/importlib.metadata.html#entry-points.
  entry_points
  if version_info >= (3, 10)
  else lambda *, group: entry_points().get(group, ())
)


class Index(RedirectView):
  def get_redirect_url(self, *args, **kwargs):
    if 'django.contrib.auth' in settings.INSTALLED_APPS \
    and not self.request.user.is_authenticated:
      return settings.LOGIN_URL
    else:
      return settings.LOGIN_REDIRECT_URL
    # def get_redirect_url
  # class Index


urlpatterns = [
  path('', Index.as_view(), name='index'),
]


# Lisää kiinteät näkymät.
for entry_point in _entry_points(group='django.nakymat'):
  try:
    moduuli = entry_point.load()
  except (ImportError, AttributeError):
    pass
  else:
    urlpatterns.append(
      path(
        entry_point.name + '/',
        include(moduuli)
      )
    )
    # else
  # for entry_point in _entry_points


# Lisää asennetut osoitteistot.
for entry_point in _entry_points(group='django.osoitteisto'):
  urlpatterns.append(
    path(
      entry_point.name + '/',
      include((entry_point.module_name, entry_point.name)),
    )
  )
  # for entry_point in _entry_points



# -*- coding: utf-8 -*-

import os

from django.core.asgi import get_asgi_application

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "protoni.palvelin")


django = get_asgi_application()

try:
  from pistoke.kasittelija import WebsocketKasittelija
except ModuleNotFoundError:
  async def websocket(*args):
    raise NotImplementedError('Pistoke-pakettia ei ole asennettu!')
else:
  websocket = WebsocketKasittelija()


async def application(scope, receive, send):
  if scope['type'] == 'http':
    return await django(scope, receive, send)
  elif scope['type'] == 'websocket':
    return await websocket(scope, receive, send)
  else:
    raise NotImplementedError(f'Tuntematon pyyntö {scope["type"]}')
  # async def application



# -*- coding: utf-8 -*-

import whitenoise
del whitenoise

if CONFIG('WHITENOISE_RUNSERVER', cast=bool, default=False):
  INSTALLED_APPS.insert(
    0, 'whitenoise.runserver_nostatic'
  )
  MIDDLEWARE[1:1] = [
    'whitenoise.middleware.WhiteNoiseMiddleware',
  ]

STATICFILES_STORAGE = (
  'whitenoise.storage'
  '.CompressedManifestStaticFilesStorage'
)



# -*- coding: utf-8 -*-

import celery
del celery

CELERY_BROKER_URL = CONFIG(
  'CELERY_BROKER_URL', default='redis://localhost:6379/0'
)
CELERY_RESULT_BACKEND = CONFIG(
  'CELERY_BROKER_URL', default='redis://localhost:6379/0'
)

INSTALLED_APPS += ['celery']



# -*- coding: utf-8 -*-

'''
Sentry-SDK-käyttöönotto. Edellyttää seuraavien muuttujien määrityksen:
- SENTRY_DSN: Sentry-palvelimen yhteysosoite
- SENTRY_PAKETTIVERSIO: sen Pip-paketin nimi, jonka versionumero
  ilmoitetaan Sentrylle.
'''
import logging

import sentry_sdk
from sentry_sdk.integrations.django import DjangoIntegration
from sentry_sdk.integrations.logging import LoggingIntegration
from sentry_sdk.scrubber import EventScrubber, DEFAULT_DENYLIST

# Poimi DSN-yhteysosoite.
try:
  dsn = CONFIG('SENTRY_DSN')
except UndefinedValueError:
  raise ImportError

# Poimi sen paketin nimi, jonka versionumero ilmoitetaan Sentrylle.
try:
  paketti = CONFIG('SENTRY_PAKETTIVERSIO')
except UndefinedValueError:
  del dsn
  raise ImportError

# Poimi paketin versionumero.
from importlib.metadata import PackageNotFoundError, version
try:
  versio = version(paketti)
except PackageNotFoundError:
  del dsn
  raise ImportError
finally:
  del paketti
  del PackageNotFoundError, version

# Alusta Sentry-määritys.
# Ks. https://docs.sentry.io/platforms/python/guides/django/
sentry_sdk.init(
  dsn=dsn,
  release=versio,
  integrations=[
    LoggingIntegration(
      level=logging.INFO,
      event_level=logging.WARNING,
    ),
    DjangoIntegration(),
  ],
  send_default_pii=CONFIG(
    'SENTRY_SEND_DEFAULT_PII',
    cast=bool,
    default=False,
  ),
  event_scrubber=EventScrubber(
    denylist=[
      *DEFAULT_DENYLIST,
      *CONFIG(
        'SENTRY_DENYLIST',
        cast=lambda x: filter(None, x.split(',')),
        default='',
      ),
    ],
  ),
)
del logging
del sentry_sdk, DjangoIntegration, LoggingIntegration
del EventScrubber, DEFAULT_DENYLIST
del dsn, versio
